<?php

namespace AIAssistantTenWeb;

class Library {
  public static function ai_button() {
    static $cache = null;

    if($cache === null) {
      $cache = true;
    } else {
      return;
    }
    ?>
    <div id="twai-button-cont" class="twai-button-cont twai-inline">
      <button id="twai-button" type="button" class="button button-primary button-large twai-button twai-button-disabled twai-button-green">
        <span class="twai-button-text"><?php echo esc_html__( '10Web AI', 'ai-assistant-tenweb' ); ?></span>
        <div class="twai-loader"></div>
        <div class="twai-list-container">
          <div class="twai-list-container-content">
            <div class="twai-list-header">
              <p class="twai-list-header-title"><?php echo esc_html__( 'Ask 10Web AI to:', 'ai-assistant-tenweb' ); ?></p>
              <span class="twai-how_to_use-button"><?php echo esc_html__( 'How to use', 'ai-assistant-tenweb' ); ?></span>
            </div>
            <div class="twai-list-empty-info">
              <p><?php echo esc_html__( 'Please select a text/tittle to use AI Assistant.', 'ai-assistant-tenweb' ); ?></p>
            </div>
            <div class="twai-list-item-content">
              <div class="twai-list-item-content-layout"></div>
              <?php
              foreach ( self::actions_list() as $key => $item ) {
                ?>
              <div class="twai-list-item" data-value="<?php echo esc_attr($key); ?>">
                <span class="twai-list-item-title"><?php echo esc_html($item['title']); ?></span>
                <span class="twai-list-item-description"><?php echo esc_html($item['description']); ?></span>
              </div>
                <?php
              }
              ?>
            </div>
            <?php self::word_usage(); ?>
          </div>
        </div>
      </button>
    </div>
    <?php
    self::how_to_use();
  }

  /**
   * Word usage container.
   *
   * @return void
   */
  public static function word_usage() {
    $limitation = \AIAssistantTenWeb\Utils::get_limitation();
    if ( !empty($limitation) && !empty($limitation['planLimit']) )  {
      if ( empty($limitation['alreadyUsed']) ) {
        $limitation['alreadyUsed'] = 0;
      }
      ?>
      <div class="twai-list-footer">
        <div class="twai-list-footer-part">
          <?php echo sprintf( __('<b>Word usage</b> %s of %s', 'ai-assistant-tenweb' ), '<span id="twai_alreadyUsed">' . $limitation['alreadyUsed'] . '</span>', '<span id="twai_planLimit">' . $limitation['planLimit'] . '</span>' ); ?>
        </div>
        <div class="twai-list-footer-part">
          <a target="_blank" href="<?php echo esc_url(TENWEB_DASHBOARD . '/ai-assistant?open=livechat'); ?>"><?php echo esc_html__( 'Upgrade', 'ai-assistant-tenweb' ); ?></a>
        </div>
      </div>
      <?php
    }
  }

  public static function simple_ai_button(){
    // button without action list
    static $cache = null;

    if($cache === null) {
      $cache = true;
    } else {
      return;
    }
    ?>
      <script id="twai-simple-button-template" type="text/html">
          <div class="twai-button-cont twai-simple-button-cont twai-inline">
              <button type="button"
                      class="button button-primary button-large twai-button twai-simple-button twai-button-disabled twai-button-green">
                  <span class="twai-button-text"></span>
                  <div class="twai-loader"></div>
                  <div class="twai-empty"></div>
              </button>
          </div>
      </script>
    <?php
  }

  /**
   * Notification popup template.
   *
   * @return void
   */
  public static function popup_template() {
    static $cache = null;

    if($cache === null) {
      $cache = true;
    } else {
      return;
    }

    ?>
    <div class="twai-popup-layout twai-hidden"></div>
    <div class="twai-popup-notif twai-hidden">
      <span class="twai-popup-close"></span>
      <h3 class="twai-popup-notif-title"></h3>
      <p class="twai-popup-notif-text"></p>
      <a class="twai-popup-notif-button"></a>
    </div>
    <?php
  }

  /**
   * Returns the available actions list.
   *
   * @return string[]
   */
  public static function actions_list() {
    return array(
      'outline' => array('title' => 'Generate an outline from a title', 'description' => 'Create an outline for your article based on the title.'),
      'intro' => array('title' => 'Generate an introduction from a title', 'description' => 'Compose an opening paragraph for your article.'),
      'paragraph' => array('title' => 'Generate a paragraph', 'description' => 'Create a paragraph from title or select a paragraph to prolong it.'),
      'conclusion' => array('title' => 'Generate a conclusion paragraph', 'description' => 'Select a paragraph to generate a conclusion for your article.'),
      'paraphrase' => array('title' => 'Paraphrase a paragraph', 'description' => 'Rewrite the selected paragraph of your article to improve it.'),
    );
  }

  public static function how_to_use() {
    ?>
    <div class="twai-how_to_use-layout twai-hidden" style="display: none"></div>
    <div class="twai-how_to_use-container twai-hidden" style="display: none">
      <div class="twai-how_to_use-title"><?php echo esc_html__( '10Web AI for writing and editing', 'ai-assistant-tenweb' ); ?></div>
      <div class="twai-how_to_use-steps">
        <div class="twai-how_to_use-steps-item"><span>1</span><?php echo esc_html__( 'Select the title or a specific section.', 'ai-assistant-tenweb' ); ?></div>
        <div class="twai-how_to_use-steps-item"><span>2</span><?php echo esc_html__( 'Press the “10Web AI”', 'ai-assistant-tenweb' ); ?></div>
        <div class="twai-how_to_use-steps-item"><span>3</span><?php echo esc_html__( 'Choose from the available options.', 'ai-assistant-tenweb' ); ?></div>
      </div>
      <div class="twai-how_to_use-content">
        <div class="twai-how_to_use-content-left">
          <div class="twai-how_to_use-list-item"><?php echo esc_html__( 'Ask AI to create an outline for your article based on the title.', 'ai-assistant-tenweb' ); ?></div>
          <div class="twai-how_to_use-list-item"><?php echo esc_html__( 'Compose an opening paragraph for your article based on title.', 'ai-assistant-tenweb' ); ?></div>
          <div class="twai-how_to_use-list-item"><?php echo esc_html__( 'Create a new paragraph from the title or select a paragraph to prolong it.', 'ai-assistant-tenweb' ); ?></div>
          <div class="twai-how_to_use-list-item"><?php echo esc_html__( 'Let AI rewrite the selected paragraph of your article to improve it.', 'ai-assistant-tenweb' ); ?></div>
          <div class="twai-how_to_use-list-item"><?php echo esc_html__( 'Select a paragraph to generate a conclusion paragraph for your article.', 'ai-assistant-tenweb' ); ?></div>
        </div>
        <div class="twai-how_to_use-content-right"></div>
      </div>
      <div class="twai-how_to_use-button-row">
        <span class="taa-green-button twai-how_to_use-popup-button"><?php echo esc_html__( 'GOT IT', 'ai-assistant-tenweb' ); ?></span>
      </div>
    </div>
    <?php
  }
}